package dev.shadowsoffire.apotheosis.adventure.affix.salvaging;

import com.mojang.datafixers.kinds.Applicative;
import dev.shadowsoffire.placebo.json.ItemAdapter;
import com.mojang.datafixers.kinds.App;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import com.google.common.base.Preconditions;
import com.mojang.serialization.Codec;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.Tag;
import net.minecraft.nbt.NbtOps;
import net.minecraft.network.FriendlyByteBuf;
import com.mojang.serialization.DynamicOps;
import net.minecraft.util.GsonHelper;
import com.mojang.serialization.JsonOps;
import com.mojang.datafixers.util.Pair;
import com.google.gson.JsonObject;
import net.minecraft.world.level.Level;
import net.minecraft.core.RegistryAccess;
import dev.shadowsoffire.apotheosis.Apoth;
import net.minecraft.world.item.crafting.RecipeType;
import net.minecraft.world.item.crafting.RecipeSerializer;
import net.minecraft.world.item.ItemStack;
import java.util.List;
import net.minecraft.world.item.crafting.Ingredient;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.Container;
import net.minecraft.world.item.crafting.Recipe;

public class SalvagingRecipe implements Recipe<Container>
{
    protected final ResourceLocation id;
    protected final Ingredient input;
    protected final List<OutputData> outputs;
    
    public SalvagingRecipe(final ResourceLocation id, final List<OutputData> outputs, final Ingredient input) {
        this.id = id;
        this.outputs = outputs;
        this.input = input;
    }
    
    public boolean matches(final ItemStack stack) {
        return this.input.test(stack);
    }
    
    public Ingredient getInput() {
        return this.input;
    }
    
    public List<OutputData> getOutputs() {
        return this.outputs;
    }
    
    public ResourceLocation m_6423_() {
        return this.id;
    }
    
    public RecipeSerializer<?> m_7707_() {
        return (RecipeSerializer<?>)Serializer.INSTANCE;
    }
    
    public RecipeType<?> m_6671_() {
        return (RecipeType<?>)Apoth.RecipeTypes.SALVAGING;
    }
    
    @Deprecated
    public ItemStack m_8043_(final RegistryAccess regs) {
        return ItemStack.f_41583_;
    }
    
    @Deprecated
    public boolean m_5818_(final Container pContainer, final Level pLevel) {
        return false;
    }
    
    @Deprecated
    public ItemStack m_5874_(final Container pContainer, final RegistryAccess regs) {
        return ItemStack.f_41583_;
    }
    
    @Deprecated
    public boolean m_8004_(final int pWidth, final int pHeight) {
        return false;
    }
    
    public static class Serializer implements RecipeSerializer<SalvagingRecipe>
    {
        public static final Serializer INSTANCE;
        
        public SalvagingRecipe fromJson(final ResourceLocation id, final JsonObject obj) {
            final List<OutputData> outputs = (List<OutputData>)OutputData.LIST_CODEC.decode((DynamicOps)JsonOps.INSTANCE, (Object)GsonHelper.m_13933_(obj, "outputs")).result().get().getFirst();
            final Ingredient input = Ingredient.m_43917_(obj.get("input"));
            return new SalvagingRecipe(id, outputs, input);
        }
        
        public SalvagingRecipe fromNetwork(final ResourceLocation id, final FriendlyByteBuf buf) {
            final List<OutputData> outputs = (List<OutputData>)OutputData.LIST_CODEC.decode((DynamicOps)NbtOps.f_128958_, (Object)buf.m_130260_().m_128423_("outputs")).result().get().getFirst();
            final Ingredient input = Ingredient.m_43940_(buf);
            return new SalvagingRecipe(id, outputs, input);
        }
        
        public void toNetwork(final FriendlyByteBuf buf, final SalvagingRecipe recipe) {
            final Tag outputs = OutputData.LIST_CODEC.encodeStart((DynamicOps)NbtOps.f_128958_, (Object)recipe.outputs).get().left().get();
            final CompoundTag netWrapper = new CompoundTag();
            netWrapper.m_128365_("outputs", outputs);
            buf.m_130079_(netWrapper);
            recipe.input.m_43923_(buf);
        }
        
        static {
            INSTANCE = new Serializer();
        }
    }
    
    public static class OutputData
    {
        public static Codec<OutputData> CODEC;
        public static final Codec<List<OutputData>> LIST_CODEC;
        ItemStack stack;
        int min;
        int max;
        
        OutputData(final ItemStack stack, final int min, final int max) {
            this.stack = stack;
            this.min = min;
            this.max = max;
            Preconditions.checkArgument(max >= min);
            this.stack.m_41764_(1);
        }
        
        public ItemStack getStack() {
            return this.stack;
        }
        
        public int getMin() {
            return this.min;
        }
        
        public int getMax() {
            return this.max;
        }
        
        static {
            OutputData.CODEC = (Codec<OutputData>)RecordCodecBuilder.create(inst -> inst.group((App)ItemAdapter.CODEC.fieldOf("stack").forGetter(d -> d.stack), (App)Codec.intRange(0, 64).fieldOf("min_count").forGetter(d -> d.min), (App)Codec.intRange(0, 64).fieldOf("max_count").forGetter(d -> d.max)).apply((Applicative)inst, OutputData::new));
            LIST_CODEC = Codec.list((Codec)OutputData.CODEC);
        }
    }
}
